############################################################################
### List of all functions contained in this file:
###  Details of what the functions do are with the function definition
############################################################################
## impute_CATIE_variables=function( CATIE, seed, cur.visit.id, print.flag=F, 
##				 id.var=id.var, variables2impute, 
##				 variablesNoMiss, END=FALSE )
## define_impute_formulas = function(CATIE.imp, vars.NA, p, variablesNoMiss)
## make_binary_side_effect_CATIE = function(CATIE, cur.visit.id,
##			      sym_var="EPSMEAN",bsym_var="bEPSMEAN")
## clean_up_side_effects_CATIE= function(CATIE, cur.visit.id, END){
## CATIE_get_bounds = function( CATIE, vars.list )
## CATIE_impute_stage2_treatment=function( CATIE )
## CATIE_impute_PANSS=function( CATIE, cur.visit.id, id.var, END=TRUE,
##			    time_varying_PANSS_predictors, SCH=TRUE,  
##  	  	            variablesNoMiss, seed )
## create_predictors4pan = function(CATIE.local,id.var,predictorsNoNest,
##		      	              END=END,SCH=SCH,predictorsNest)
##
## CATIE_impute_stage_transition = function(CATIE,variablesNoMiss,seed
###########################################################################

###########################################################################
## This function imputes CATIE variables: variables2impute
## 	using variablesNoMiss as predcitors.
###########################################################################
impute_CATIE_variables=function( CATIE, seed, cur.visit.id, print.flag=F, 
				 id.var=id.var, variables2impute, 
				 variablesNoMiss, END=FALSE ){

 ## Creates a binary variable for side effect that indicates if the symptom is
 ## present or not.  Side-effect variable is semi-continuous variable, 
 ## so break it up into binary and contiuous portion for iputations
 if( sum(variables2impute == paste("END_EPSMEAN",cur.visit.id,sep=".") ) != 0){
  CATIE = make_binary_side_effect_CATIE( CATIE=CATIE, cur.visit.id,
		      sym_var="END_EPSMEAN",bsym_var="END_bEPSMEAN" )
  		      variables2impute = c(variables2impute, 
		      paste("END_bEPSMEAN",cur.visit.id,sep="."))
 } 
 if( sum(variables2impute == paste("EPSMEAN",cur.visit.id,sep=".") ) != 0){
  CATIE = make_binary_side_effect_CATIE( CATIE=CATIE, cur.visit.id,
		      sym_var="EPSMEAN",bsym_var="bEPSMEAN" )
  variables2impute = c(variables2impute, 
  		   paste("bEPSMEAN",cur.visit.id,sep="."))
 } 

 ## Separate CATIE into data set involved in imputations, variables that either
 ##   need to be imputed or variables  used as predictors for other imputaions
 ## And a data set of variables not involved in the imputations at this stage
 ##  all variables that were collected after cur.visit.id
 ### if need to imputed EPSMEAN, make sure that bEPSMEAN is added to the list
 ### to impute as well.
 CATIE.imp = CATIE[,unique(c(variablesNoMiss,variables2impute))]
 CATIE.other = CATIE[,unique( c(id.var,
  	         setdiff(names(CATIE),c(variablesNoMiss,variables2impute)) ))]

 ## Make predictor matrix so only variables collected prior current visit are 
 ## used as predictors
 p = matrix(0,nrow=length(names(CATIE.imp)),ncol=length(names(CATIE.imp)))
 colnames(p) = rownames(p) = names(CATIE.imp)
 p[variables2impute,variablesNoMiss] = 1
 diag(p) = 0
 
 ## Create formulas for mice to impute missing values
 temp.list = define_impute_formulas( CATIE.imp, vars.NA=variables2impute, p, 
  		    	      variablesNoMiss=c(id.var,variablesNoMiss) ) 
 impute.formulas = temp.list$impute.formulas 
 if(length(which(names(impute.formulas)==id.var)) > 0){
  impute.formulas = impute.formulas[-which(names(impute.formulas)==id.var)] 
 }
 CATIE.imp = temp.list$CATIE.imp
 visit.order = 1:ncol(CATIE.imp)
 visit.order = visit.order[colSums(is.na(CATIE.imp))>0]
 for(ii in names(CATIE.imp)){
  if(is.factor(CATIE.imp[,ii])){
   CATIE.imp[,ii] = factor(CATIE.imp[,ii])
  }
  if( sum(!is.na(CATIE.imp[,ii])) == 1){
   CATIE.imp[is.na(CATIE.imp[,ii]),ii] = CATIE.imp[!is.na(CATIE.imp[,ii]),ii]
  }
 }

 ### Use MICE to impute missing scheduled variabels at this visit
 tmi = mice(data=CATIE.imp, m = 1, method=impute.formulas, 
       predictorMatrix=p, visitSequence=visit.order, printFlag=F, 
       diagnostics=F, seed=seed, 
       defaultImputationMethod=c("norm","logreg","polyreg"))
 CATIE.imp = complete(tmi)

 ### add the id variable back in so CATIE can be merged back in
 CATIE.imp = cbind(CATIE[,id.var],CATIE.imp)
 names(CATIE.imp)[1] = "CATIEID"
 CATIE = merge(CATIE.other, CATIE.imp)
 if(sum(regexpr("EPSMEAN",variables2impute)>0)>0){
  CATIE = clean_up_side_effects_CATIE( CATIE=CATIE, cur.visit.id, END=END )
 }
 ##################################################################
 ### Make sure that all the variables are within the possible bounds
 ##################################################################
 temp = CATIE_get_bounds(vars.list=c(variablesNoMiss,variables2impute))
 upper.bounds = temp[["upper.bounds"]]  
 lower.bounds = temp[["lower.bounds"]]  
 for( ii in names(upper.bounds[!is.na(upper.bounds)]) ){
  if( sum(CATIE[,ii] > upper.bounds[ii]) > 0 ){
   CATIE[CATIE[,ii] > upper.bounds[ii],ii] = upper.bounds[ii]
  }
 }
 for(ii in names(lower.bounds[!is.na(lower.bounds)]) ){
  if( sum(CATIE[,ii] < lower.bounds[ii]) > 0 ){
   CATIE[CATIE[,ii] < lower.bounds[ii],ii] = lower.bounds[ii]
  }
 }
 return(CATIE)
}

##############################################################################
##### The function returns a vector containing the imputation method     #####
#####   for all variables in the CATIE data set.  For variables 	 #####
#####   involving interactions with variables, the vector entry 	 #####
#####	contains a string with the formula for the prediction		 #####
#####   For variables not involving interactions the element contains    #####
#####	"norm" for continuous variables, "logreg" for binary variables   #####
#####	and "polyreg" for categorical variables with more than 2 	 #####
#####	categories    	  	      		     	       		 #####
##############################################################################
#####  CATIE: a CATIE data set as input to CATIE_impute 		 #####
#####  vars.NA: a vector of length ncol(CATIE) indicating the number 	 #####
#####  		of missing values present for each variable   		 #####
#####  p: the prediction matrix to be input into mice			 #####
##############################################################################
define_impute_formulas = function(CATIE.imp, vars.NA, p, variablesNoMiss){

 temp.NA = names(CATIE.imp)[colSums(is.na(CATIE.imp))>0]
 vars.NA = unique(c(vars.NA,temp.NA))
 impute.formulas = rep("",dim(CATIE.imp)[2])
 names(impute.formulas) = names(CATIE.imp)

 # for each of the variables that need to be imputed loop throw and make the 
 # forumla to input input MICE
 for( i in vars.NA ){
  this.visit.id = ""
  ## get the visit number of this variable
  if( substr( i, start=nchar(i)-3, stop=nchar(i)-3) == "."){
    this.visit.id = as.numeric(substr(i,start=nchar(i)-2,stop=nchar(i)))
  }
  if( substr( i, start=nchar(i)-4, stop=nchar(i)-4) == "."  ){
    this.visit.id = as.numeric(substr(i,start=nchar(i)-3,stop=nchar(i)))
  }

  if( (sum(is.na(CATIE.imp[,i]))==0) & (sum(vars.NA==i)>0) ){
   ### if no variables with missing data print warning and leave function
   print(paste("Variable:",i,"something wrong here, nothing to impute..."))
  }else if( (sum(is.na(CATIE.imp[,i]))==0) ){
   ### if there is nothing to impute, formula is be ""
   impute.formulas[i] = ""
  }else if( length(unique(CATIE.imp[,i]))==2 ){
   ### This means there is only one uniue non missing value
   ### fill in all the missing values with this one value and move on...
   t = unique(CATIE.imp[,i])[!is.na(unique(CATIE.imp[,i]))]
   CATIE.imp[is.na(CATIE.imp[,i]),i] = t
   impute.formulas[i] = ""
  } else {
   # Use norm for continuous variables
   if(!is.factor(CATIE.imp[,i])){  
      impute.formulas[i] = "norm" 
   }else{
    # Use logreg or polyreg as appropriate
    CATIE.imp[,i] = factor(CATIE.imp[,i])
    if( nlevels(CATIE.imp[,i]) == 2 ){  
     impute.formulas[i] = "logreg"
    }else{  
     impute.formulas[i] = "polyreg"
    }
   }
  }# close missing values present loop
  #  print(impute.formulas[i])
 }# close for loop over variables
 return(list(impute.formulas=impute.formulas,CATIE.imp=CATIE.imp))
} # close make impute formulas function loop

###########################################################################
## The movement disorder scores in CATIE have a semi-continuous distribution
##  Most people have no movement disorder, i.e. a value of 0 on the score
##  Make a binary variable that indicated if an individual has a movement
##  disorder or not in addition to the continuous valued score for those
##  who have a movement disorder
###########################################################################
make_binary_side_effect_CATIE = function(CATIE, cur.visit.id,
			      sym_var="EPSMEAN",bsym_var="bEPSMEAN"){
 temp.levels = c("No Symptoms","Symptoms")
 sym_var = paste(sym_var,cur.visit.id,sep=".")
 bsym_var = paste(bsym_var,cur.visit.id,sep=".")
 CATIE[,bsym_var] = factor(!is.na(CATIE[,sym_var])& CATIE[,sym_var]!=0)
 temp.unique = sort(unique(as.numeric(CATIE[,bsym_var])))
 levels(CATIE[,bsym_var]) = temp.levels[temp.unique ]
 CATIE[is.na(CATIE[,sym_var]),bsym_var] = NA
 CATIE[!is.na(CATIE[,sym_var])&CATIE[,sym_var]==0,sym_var] = NA
 return(CATIE)
}
###########################################################################
##  Several CATIE variables have upper and lower bounds, this is used for 
##   ensuring the bounds are met in the imputed data
###########################################################################
CATIE_get_bounds = function( CATIE, vars.list ){

 # set up the bounds
 upper.bounds = rep(NA,length(vars.list))
 names(upper.bounds) = vars.list
 lower.bounds = upper.bounds

 t = regexpr("BMI",vars.list)>0
 lower.bounds[t] = 15.0
 upper.bounds[t] = 65.0

 t.PANSS =  regexpr("PANSS",vars.list)
 upper.bounds[t.PANSS>0] = 210
 lower.bounds[t.PANSS>0] = 30

 # these are greater than 0
 # "MEDAD03","QOL_TOT", "YRS_PRES"
 for(jj in c("MEDAD03","QOL_TOT","YRS_PRES") ){
  t.temp =  regexpr(jj,vars.list,fixed=T)
  lower.bounds[ t.temp>0 ] = 0
 }
 t.temp =  regexpr("MEDAD03",vars.list)
 upper.bounds[t.temp>0] = 100
 
 t = regexpr("EPSMEAN",vars.list,fixed=T) > 0 & 
   regexpr("b",vars.list,fixed=T) < 0
 upper.bounds[t] = 4.0
 lower.bounds[t] = 0

 t = regexpr("QOL_TOT",vars.list,fixed=T) > 0
 upper.bounds[t] = 6.0

 return(list(upper.bounds=upper.bounds,lower.bounds=lower.bounds))
}

###############################################################################
####	Impute stage 2 treatment at the first follow-up visit	              
###############################################################################
CATIE_impute_stage2_treatment=function(CATIE){
 ## indicies of people in each of the randomization arms that have missing 
 ## stage 2 treatment
 temp.index.miss.stage2 = CATIE$SWITCH=="SWITCHED" & is.na(CATIE$TREATs2)
 temp.index.eff = temp.index.miss.stage2 & !is.na(CATIE$STAGE2_ARM) & 
 		  CATIE$STAGE2_ARM == "Efficacy"
 temp.index.tol = temp.index.miss.stage2 & !is.na(CATIE$STAGE2_ARM) & 
 		  CATIE$STAGE2_ARM == "Tolerability"

 Tol.t2 = c("Olanzapine","Risperidone","Ziprasidone")
 Eff.t2 = c("Olanzapine","Risperidone","Clozapine")
 ### if there is no one missing treatment then return CATIE as is
 if( sum(temp.index.miss.stage2)==0 ){
  print("No missing stage two treatment")
  return(CATIE)
 }
 ## for all possible treatment that a person could have been randomized 
 ## 	to before phase 2
 ## loop through so people cannot get randomized to the same treatment twice
 for( ii in  c("Olanzapine","Quetiapine","Risperidone","Perphenazine") ){

  ## These people recieved treatment ii in stage 1
  temp.treat.ii = ( CATIE[,"TREATs1"] == ii )
  
  if( ii != "Perphenazine" & ii != "Quetiapine" ){

   # for people who had either olanzapine or risperidone and diccontinued, make
   # sure not rerandomized to those medications again.
   ### Efficacy arm
   temp.treat.now = Eff.t2[-which(Eff.t2==ii)]   
   CATIE[temp.index.eff&temp.treat.ii,"TREATs2"] = temp.treat.now[
			   sample(size=sum(temp.treat.ii[temp.index.eff]),
			   x=1:2,replace=T,prob=c(.5,0.5)) ]
   ### Tolerability arm
   temp.treat.now = Tol.t2[-which(Tol.t2==ii)]   
   CATIE[temp.index.tol&temp.treat.ii,"TREATs2"] = temp.treat.now[
			   sample(size=sum(temp.treat.ii[temp.index.tol]),
			   x=1:2,replace=T,prob=c(.5,0.5)) ]
  }else{
   # As CATIE protocol randomization probabilities for clozapine 
   # and ziprasidone are higher than the other stage 2 treatments
   ### Efficacy arm
   temp.treat.now = Eff.t2
   CATIE[temp.index.eff&temp.treat.ii,"TREATs2"] = temp.treat.now[
			 sample(size=sum(temp.treat.ii[temp.index.eff]),
			 x=1:3,replace=T,prob=c(0.50,0.25,0.25)) ]
   ###              Tolerability arm
   temp.treat.now = Tol.t2
   CATIE[temp.index.tol&temp.treat.ii,"TREATs2"] = temp.treat.now[
			 sample(size=sum(temp.treat.ii[temp.index.tol]),
			 x=1:3,replace=T,prob=c(.50,0.25,0.25)) ]
   }
 }
 CATIE$TREAT.2 = CATIE$TREATs2
 CATIE$TREAT.2 = factor(CATIE$TREAT.2)
 levels(CATIE$TREAT.2) = c(levels(CATIE$TREAT.2),"Perphenazine","Quetiapine")
 CATIE$TREAT.2[CATIE$SWITCH=="STAYED"] = CATIE[CATIE$SWITCH=="STAYED",
 				       "TREATs1"]
 return(CATIE)
}

###########################################################################
## Make sure after imputations that binary indicator (bEPSMEAN) for no 
## symptoms matches with the continuous score (EPSMEAN)
###########################################################################
clean_up_side_effects_CATIE= function(CATIE, cur.visit.id, END){
 if(END){
  CATIE[!is.na(CATIE[,paste("END_bEPSMEAN",cur.visit.id,sep=".")])&
	  CATIE[,paste("END_bEPSMEAN",cur.visit.id,sep=".")]=="No Symptoms",
		   	paste("END_EPSMEAN",cur.visit.id,sep=".")] = 0
  CATIE[!is.na(CATIE[,paste("END_EPSMEAN",cur.visit.id,sep=".")])&
	  CATIE[,paste("END_EPSMEAN",cur.visit.id,sep=".")]==0,
	   	paste("END_bEPSMEAN",cur.visit.id,sep=".")] = "No Symptoms"
  }else{
   CATIE[!is.na(CATIE[,paste("bEPSMEAN",cur.visit.id,sep=".")])&
	CATIE[,paste("bEPSMEAN",cur.visit.id,sep=".")]=="No Symptoms",
			paste("EPSMEAN",cur.visit.id,sep=".")] = 0
   CATIE[!is.na(CATIE[,paste("EPSMEAN",cur.visit.id,sep=".")])&
	   CATIE[,paste("EPSMEAN",cur.visit.id,sep=".")]==0,
		paste("bEPSMEAN",cur.visit.id,sep=".")] = "No Symptoms"
  }
 return(CATIE)
}

##################################################################
### Impute missing PANSS variable with mixed effect model
##################################################################
CATIE_impute_PANSS=function( CATIE, cur.visit.id, id.var, END=TRUE,
			    time_varying_PANSS_predictors, SCH=TRUE,  
  	  	            variablesNoMiss, seed ){

 ## Keep track of how many visits have "gone by" 
 cur.visit.ids = 0:cur.visit.id

 # get all the time-varying variables with the visit id attached that will be
 #  used in the imputation. Either as predictors or "PANSSTOT" because it 
 #  needs to be imputed
 all_varying = NULL
 for(iv in cur.visit.ids){
  all_varying = c(all_varying,
  	      paste(c(time_varying_PANSS_predictors,"PANSSTOT"),iv,sep="." )) 
 }
 all_varying = unique(all_varying)
 variablesPredPANSSNoNest = setdiff(variablesNoMiss,all_varying)
 if( which(time_varying_PANSS_predictors =="MEDAD03") & 
     					 sum(names(CATIE)=="MEDAD03.0") == 0){
  CATIE$MEDAD03.0 = 0
 }
 if( which(time_varying_PANSS_predictors =="TREAT") & 
     					 sum(names(CATIE)=="TREAT.0") == 0){
  CATIE$TREAT.0 = "No Trt Yet"
  CATIE$TREAT.0 = factor(CATIE$TREAT.0)
 }
 # Separate out the data into that to be used in the imputation strategy 
 #    at this point and variables in the future
 CATIE.imp = CATIE[,unique(c(variablesPredPANSSNoNest,
		all_varying,"STAGE2_ARM"))]
 CATIE.other = CATIE[,unique( c(id.var, 
 	        setdiff(names(CATIE),names(CATIE.imp))))]
 ## reshape into long format to use the mixed effect imputation model.
 CATIE.long = reshape(CATIE.imp,direction="long", varying= all_varying,
			idvar=id.var,timevar="Visit")
 
 ### CATIE.long must be sorted by ID or pan will FREAK OUT
 CATIE.long = CATIE.long[sort(CATIE.long[,id.var],index.return=TRUE)$ix,]

 ## Create the design matrix to pass into pan
 temp.list = create_predictors4pan(CATIE.long,id.var=id.var,END=END,SCH=SCH,
			predictorsNoNest=variablesPredPANSSNoNest,
			predictorsNest=time_varying_PANSS_predictors)

 ##############################################################################
 ##############################################################################
 ####  This section is copied from comments in some example code from Schafers 
 ####  who created and maintains the pan function, please see his documentation
 ####  for more detailed information.
 ##############################################################################
 ## Sigma is the parameter that specifies the covariance matrix for response.
 ## Recall that the dimension of Sigma is (r x r) where r
 ## is the number of response variables (in this case, r=1). The prior
 ## distribution for Sigma is inverted Wishart with hyperparameters a 
 ## (scalar) and Binv (r x r), where a is the imaginary degrees of freedom
 ## and Binv/a is the prior guesstimate of Sigma. The value of a must be
 ## greater than or equal to r. The "least informative" prior possible
 ## would have a=r, so here we will take a=1. As a prior guesstimate of 
 ## Sigma we will use the (r x r) identity matrix, so Binv = 1*1 = 1.
 ##############################################################################
 ## By similar reasoning we choose the prior distribution for Psi. The
 ## dimension of Psi is (r*q x r*q) where q is the number of random
 ## effects in the model (which in this case is one).
 ## The hyperparameters for Psi are c and Dinv, where c is the
 ## imaginary degrees of freedom (which must be greater than or equal to
 ## r*q) and Cinv/d is the prior guesstimate of Psi. We will take d=1
 ## and Cinv=1*1 = 1.
 ##############################################################################
 y.pan = temp.list$resp
 subj.pan = as.integer(temp.list$subj)
 pred.pan = as.matrix(temp.list$Pred)
 attr(pred.pan,"contrasts") = NULL
 attr(pred.pan,"assign") = NULL
 xcol.pan = 1:ncol(pred.pan)
 zcol.pan = which(colnames(pred.pan)=="(Intercept)")
 prior.list=list(a=1, Binv=matrix(1,nrow=1,ncol=1), c=1, 
 		      Dinv=diag(1,nrow=1,ncol=1))
 pan.out = pan(y=y.pan,subj=subj.pan,pred=pred.pan, xcol=xcol.pan,
	       zcol=zcol.pan, prior=prior.list, seed=floor(runif(1)*32000), 
	       iter=250)
 # This is just to gaurd against collinearity, delete if fine at visit 2

 CATIE.long$PANSSTOT = pan.out$y  
 # make sure in bounds
 if( sum(CATIE.long$PANSSTOT<30) > 0 ){
  CATIE.long$PANSSTOT[CATIE.long$PANSSTOT<30] = 30
 }
 if( sum(CATIE.long$PANSSTOT>210) > 0 ){
  CATIE.long$PANSSTOT[CATIE.long$PANSSTOT<210] = 210
 }

 # Put the data back into wide format
 CATIE.impd = reshape(CATIE.long,direction="wide",idvar=id.var,timevar="Visit",
 	      v.names = c(time_varying_PANSS_predictors,"PANSSTOT"))
 CATIE.impd = CATIE.impd[sort(CATIE.impd[,id.var],index.return=TRUE)$ix,]
 CATIE.other = CATIE.other[sort(CATIE.other[,id.var],index.return=TRUE)$ix,]
 CATIE = merge(CATIE.other,CATIE.impd,by="CATIEID")

 if( sum(names(CATIE)=="MEDAD03.0")>0 ){
  CATIE = CATIE[,-which(names(CATIE)=="MEDAD03.0")]
 }
 if( sum(names(CATIE)=="TREAT.0")>0 ){
  CATIE = CATIE[,-which(names(CATIE)=="TREAT.0")]
 }
 return(CATIE)
}

###########################################################################
### This function makes everything numeric because pan needs all predictors
###   and outcomes numeric
###########################################################################
create_predictors4pan = function(CATIE.local,id.var,predictorsNoNest,
		      	              END=END,SCH=SCH,predictorsNest){
# This assumes there are no missing values in this data set other than PANSS
# ie that only the subjects that need to be imputed are in CATIE.local  

# predictorsNoNest = variablesPredPANSSNoNest
# predictorsNest = time_varying_PANSS_predictors

 ### with the results call pan such as
 # temp.list = create_predictors4pan(CATIE.local)
 # pan.out = pan(temp.list$resp,temp.list$subj,
 #             pred=temp.list$Pred, xcol=1:dim(X)[2] , zcol=1,
 #             prior=prior.list, seed=1,iter=1000);

 ### This is ordered categorical so just convert to numeric
 CATIE.pan = CATIE.local
 
 if( sum(regexpr("CS16",names(CATIE.pan)) > 0)>0 ){
  CATIE.pan[,"CS16"] = as.numeric(CATIE.local[,"CS16"])  
 }
 ### Make Stage 2_Arm a variable we can use to nest end of stage predictors in 
 if(END & SCH){
  if( sum(regexpr("STAGE2_ARM",names(CATIE.pan)) > 0)>0 ){
   temp.levels = levels(CATIE.pan$STAGE2_ARM)
   CATIE.pan$STAGE2_ARM = as.numeric(CATIE.pan$STAGE2_ARM )
   CATIE.pan$STAGE2_ARM[is.na(CATIE.pan$STAGE2_ARM)] = 3
   CATIE.pan$STAGE2_ARM = factor(CATIE.pan$STAGE2_ARM,
   			  labels=c(temp.levels,"STAGE 1"))
  }
 }

 ### This is the imputation formula for PANSS
 ## predictorsNoNest - are those variabels not nested in end of phase
 ## predictorsNest - are those variables nested within SWITCH and 
 ## 		   STAGE2_ARM (which is not represented in one variable
 ##		   called STAGE2_ARM, just for this function).
 if(END){
  # if predicting end of stage PANSS nest predictors
  temp.form  = paste("~",paste(predictorsNoNest,collapse="+"),"+STAGE2_ARM*(",
  	          paste(predictorsNest,collapse="+"),")")
  CATIE.pan.mod = model.matrix(as.formula(temp.form), data=CATIE.pan)
 }else {
  # if only imputing scheduled PANSS no need to nest.
  temp.form  = paste("~",paste(predictorsNoNest,collapse="+")," + (",
  	          paste(predictorsNest,collapse="+"),")")
  CATIE.pan.mod = model.matrix(as.formula(temp.form), data=CATIE.pan)
 }

 ### return a list with things to input into pan
 ### Pred is matrix of predictors with no missing data, everythins is numeric
 ### 	   and it is in a matrix format
 ### response is PANSS in matrix form, with 1 column.
 ### subj is CATIEID as an integer.
 return(list(Pred=CATIE.pan.mod,
	resp=as.matrix(CATIE.local[,"PANSSTOT"],ncol=1),
	subj=as.integer(CATIE.local[,id.var])))
  
 ### with the results call pan such as
 # temp.list = create_predictors4pan(CATIE.local)
 # pan.out = pan(temp.list$resp,temp.list$subj,
 #             pred=temp.list$Pred, xcol=1:dim(X)[2] , zcol=1,
 #             prior=prior.list, seed=1, iter=1000);

}

###########################################################################
## Use an imputation model to impute if someone transitioned to the 
##     next treatment stage
###########################################################################
impute_CATIE_stage_transition = function( CATIE, variablesNoMiss, seed,
			      		  	 id.var=id.var ){

 ## Separate CATIE into data set involved in imputations,
 CATIE.imp = CATIE[,unique(c(variablesNoMiss,"SWITCH"))]
 CATIE.other = CATIE[,unique( c(id.var,
  	         setdiff(names(CATIE),c(variablesNoMiss,"SWITCH")) ))]

 ### Use MICE to impute missing scheduled variabels at this visit
 tmi = mice(data=CATIE.imp, m = 1, method="logreg", seed=seed, 
       			 printFlag=F, diagnostics=F, 
       			 defaultImputationMethod=c("norm","logreg","polyreg") )
 CATIE.imp = complete(tmi)
 CATIE.imp = cbind(CATIE[,id.var],CATIE.imp)
 names(CATIE.imp)[1] = "CATIEID"
 CATIE = merge(CATIE.other, CATIE.imp)
 return(CATIE)

}

